#!/opt/edge/venv/bin/python3.11

import semver

from edge.webserver.sysconf.uimod import uitype
from edge.webserver.sysconf import ph


_TARGET_VER = '0.37.58'


def _update_ver(unit_info):
    """
    :type unit_info: dict
    :rtype: None
    """
    ui_ref = unit_info['ref']
    ui_mod = ph.UiMod.create(ui_ref)
    ui_conf = ui_mod.load_conf()

    source_ver = ui_conf['ro'].get('ver')
    if source_ver is not None:
        if semver.compare(source_ver, _TARGET_VER) >= 0:
            return

    ui_conf['ro']['ver'] = _TARGET_VER
    ui_mod.save_conf(ui_conf)


def _main():
    # Must be first
    root_info = ph.RootInfoUiconf.get_info()
    ph.RootInfoUiconf.map_units_r(
        root_info,
        map_fn=_update_ver,
        unit_types=[
            uitype.UI_TYPE_DET_BUS_LANE,
            uitype.UI_TYPE_LOCATION,
            uitype.UI_TYPE_LPR,
            uitype.UI_TYPE_DET_PARKING,
            uitype.UI_TYPE_DET_RAILROAD,
            uitype.UI_TYPE_DET_REDLIGHT,
            uitype.UI_TYPE_DET_SEATBELT,
            uitype.UI_TYPE_DET_SPOT_SPEED,
            uitype.UI_TYPE_DET_SPOT_SPEED_WAGA,
            uitype.UI_TYPE_TERMINAL,
            uitype.UI_TYPE_DET_TRANSIT
        ])

    if __name__ == '__main__':
        _main()
