from edge.webserver.sysconf.uimod import uitype
from edge import eventlog
from edge.support import fs
from edge.support.mask import MaskRegion
from edge.types.exceptions import *
from edge.webserver.sysconf import ph

import edge.webserver.sysconf.edge_creator as creator


# noinspection PyClassHasNoInit
class RoadwayApi:
    @staticmethod
    def load_unit(uimod_ref: str) -> dict:
        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(uitype.UI_TYPE_ROADWAY)
        cam_dg_stats = ph.CamDgStats.create(uimod.pid)
        cam_dg_stats.test_availability()
        cam_stats_jnode = cam_dg_stats.get_stats(uimod.pid)

        uiconf = uimod.load_conf()
        ext_uiconf_pub = uiconf['rw']

        reply = {
            'type': uitype.UI_TYPE_ROADWAY,
            'conf': ext_uiconf_pub['args'],
            'stats': cam_stats_jnode
        }
        return reply

    @staticmethod
    def save_unit(uimod_ref: str, new_uiconf: dict) -> dict:
        new_uiconf_conf = new_uiconf.get('conf')
        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(uitype.UI_TYPE_ROADWAY)
        cam_dg_stats = ph.CamDgStats.create(uimod.pid)
        cam_dg_stats.test_availability()
        cam_stats_jnode = cam_dg_stats.get_stats(uimod.pid)

        def validate_uiconf():
            fs.validate_json({'args': new_uiconf_conf}, uimod.schema_path)

            try:
                # validate self-intersection of lanes
                entry_lanes = new_uiconf_conf['lanes']
                for lane_jnode in entry_lanes:
                    mask_region = MaskRegion(lane_jnode['mask'])
                    if not mask_region.test_valid():
                        eventlog.log_d('fail: ParamsError (invalid road-mask)')
                        raise ParamsError

            except (ValueError, IndexError):
                eventlog.log_d('fail: ParamsError (road-mask contains less then 3 vertexes)')
                raise ParamsError

        validate_uiconf()

        uiconf = uimod.load_conf()
        uiconf_pub = uiconf['rw']
        if uiconf_pub['args'] != new_uiconf_conf:
            uiconf_pub['args'] = new_uiconf_conf
            uirefs = ph.RootInfoUiconf.get_linked_uirefs(uimod.ref)
            creator.generate(uimod.create_backup_dirpath(), uirefs, {uimod.ref: uiconf})

        reply = {
            'type': uitype.UI_TYPE_ROADWAY,
            'conf': new_uiconf_conf,
            'stats': cam_stats_jnode
        }
        return reply
