from edge.webserver.sysconf.uimod import uitype
from edge import eventlog
from edge.support import fs
from edge.types.exceptions import *
from edge.webserver.sysconf import ph

import edge.webserver.sysconf.edge_creator as creator


class WantedApi:
    @staticmethod
    def make_detector(term_pid: str, factory_conf: dict) -> tuple[str, dict]:
        if not ph.RootInfoUiconf.validate_term(term_pid):
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        det_uimod_pid = term_pid
        det_uimod_type = factory_conf.get('type')
        det_name = factory_conf.get('name')

        if det_uimod_type != uitype.UI_TYPE_DET_WANTED or det_name is None:
            eventlog.log_d('fail: ParamsError')
            raise ParamsError

        answer = ph.RootInfoUiconf.add_detector(det_uimod_pid, det_uimod_type, det_name)
        if answer is None:
            eventlog.log_e('fail: InternalError')
            raise InternalError

        det_uimod_id, root_info = answer

        uimod = ph.UiMod(det_uimod_pid, det_uimod_type, det_uimod_id)
        uimod.install()

        uiconf = uimod.load_conf()
        uiconf['rw']['args']['name'] = det_name
        uirefs = ph.RootInfoUiconf.get_linked_uirefs(uimod.ref)
        creator.generate(uimod.create_backup_dirpath(), uirefs, {uimod.ref: uiconf})
        return uimod.ref, root_info

    @staticmethod
    def drop_detector(uimod_ref: str) -> dict:
        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(uitype.UI_TYPE_DET_WANTED)
        if uimod.load_conf(default=None) is None:
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        root_info, changed_uirefs = ph.RootInfoUiconf.del_unit_r(uimod.ref)
        creator.generate(uimod.create_backup_dirpath(), changed_uirefs)
        uimod.uninstall()

        ext_root_info = ph.RootInfoUiconf.get_ext_root_info(root_info)
        reply = {
            'info': [ext_root_info]
        }
        return reply

    @staticmethod
    def load_detector(uimod_ref: str) -> dict:
        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(uitype.UI_TYPE_DET_WANTED)

        uiconf = uimod.load_conf(default=None)
        if uiconf is None:
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        uiconf_pub = uiconf['rw']
        reply = {
            'type': uitype.UI_TYPE_DET_WANTED,
            'conf': uiconf_pub['args']
        }
        reply.update(ph.get_detector_limits_stats(uimod.pid, scene=False, vehicle=False))
        return reply

    @staticmethod
    def save_detector(uimod_ref: str, new_uiconf: dict) -> dict:
        new_uiconf_conf = new_uiconf.get('conf')

        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(uitype.UI_TYPE_DET_WANTED)

        if uimod.load_conf(default=None) is None:
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        fs.validate_json({'args': new_uiconf_conf}, uimod.schema_path)

        uiconf = uimod.load_conf()
        if uiconf['rw']['args'] != new_uiconf_conf:
            if new_uiconf_conf['name'] != uiconf['rw']['args']['name']:
                ph.RootInfoUiconf.set_unit_name(uimod.ref, new_uiconf_conf['name'])

            uiconf['rw']['args'] = new_uiconf_conf
            uirefs = ph.RootInfoUiconf.get_linked_uirefs(uimod.ref)
            creator.generate(uimod.create_backup_dirpath(), uirefs, {uimod.ref: uiconf})

        reply = {
            'type': uitype.UI_TYPE_DET_WANTED,
            'conf': new_uiconf_conf
        }
        reply.update(ph.get_detector_limits_stats(uimod.pid, scene=False, vehicle=False))
        return reply
