from typing import Final, List

from edge.support import net
from edge.webserver.sysconf import ph
from edge.webserver.sysconf.dgmod import dg
from edge.webserver.sysconf.dgmod.dgconf import IDgConfBuilder, UirefToConfMap, Uideps, IDgConfDeleter
from edge.webserver.sysconf.types.conf import IUiConfProvider
from edge.webserver.sysconf.uiconf.constroller import CtrlUiconf
from edge.webserver.sysconf.uiconf.db import DbUiconf
from edge.webserver.sysconf.uimod import uitype


def _reconfigure_es(endpoints: list[str]) -> bool:
    for _ in [1, 2, 3]:
        try:
            net.query_es_config(endpoints, tout_sec=1)
            return True
        except net.NetError:
            pass

    return False


def _mk_react_path(rule_uimod: IUiConfProvider) -> str:
    return f'/var/spool/edge/system/sc{rule_uimod.pid}-es65535-fact'


# noinspection PyUnusedLocal
def es64_create_dgconf_builder(
        target_uiref: str,
        target_uideps: Uideps,
        uiref_to_newconf_map: UirefToConfMap,
        forced: bool = False) -> list[IDgConfBuilder]:

    activate = forced

    if not activate:
        return []

    root_info = ph.RootInfoUiconf.get_info()
    tr_uirefs = ph.RootInfoUiconf.find_unit_refs_r(root_info, uitype.UI_TYPE_TRAFFIC_RULES)
    rules_uimods = list(map(ph.UiMod.create, tr_uirefs))
    return [_EsCreator64(rules_uimods)]


class _EsCreator64(IDgConfBuilder):
    def __init__(self, rules_uimods: list[IUiConfProvider]) -> None:
        self._rules_uimods: Final[list[IUiConfProvider]] = rules_uimods

        self._main_conf = None

    @property
    def dgmod(self) -> dg.DgMod:
        return dg.CTRL_ES_DGMOD

    def setup(self, uiref_to_conf_map: UirefToConfMap) -> None:
        es_new_conf = {
            'api': net.get_es_uds_endpoint(),
            'db': DbUiconf.CONF_PATH,
            'monitoring': {
                'stat-path': '/var/spool/edge/system/zabbix/es65535',
                'stat-tout': 60
            },
            'fact-react': list(map(_mk_react_path, self._rules_uimods)),
            'systest': CtrlUiconf.get_systest()
        }

        self._main_conf = es_new_conf

    def build(self) -> None:
        es_dgmod = dg.CTRL_ES_DGMOD
        es_dgmod.install()
        es_dgmod.save_main_conf(self._main_conf)
        es_dgmod.save_setupstore_conf(active=True)

    def notify(self) -> None:
        if not _reconfigure_es(self._main_conf['fact-react']):
            dg.LifeCycle.sigkill_dgmods(dg.EDGE_TYPE_EVENTSTORE)


class _EsDeleter64(IDgConfDeleter):
    def __init__(self, uimod_fa: IUiConfProvider) -> None:
        self._uimod_fa = uimod_fa

    def run(self) -> None:
        dgmod = dg.CTRL_ES_DGMOD
        main_conf = dgmod.load_main_conf()

        endpoint = _mk_react_path(self._uimod_fa)
        main_conf['fact-react'] = [x for x in main_conf['fact-react'] if x != endpoint]

        dgmod.save_main_conf(main_conf)

        if not _reconfigure_es(main_conf['fact-react']):
            dg.LifeCycle.sigkill_dgmods(dg.EDGE_TYPE_EVENTSTORE)


def es64_create_dgconf_deleter(target_uimod: IUiConfProvider) -> List[IDgConfDeleter]:
    if target_uimod.type == uitype.UI_TYPE_FACTORY:
        return [_EsDeleter64(target_uimod)]

    return []
