from typing import Final, List

from edge import eventlog
from edge.types.exceptions import NotFoundError
from edge.webserver.sysconf.dgmod import dg
from edge.webserver.sysconf.dgmod.dgconf import IDgConfDeleter,IDgConfBuilder, UirefToConfMap, Uideps
from edge.webserver.sysconf.types.conf import IUiConfProvider
from edge.webserver.sysconf.uimod import uitype


# noinspection PyUnusedLocal
def msw_create_dgconf_builder(
        target_uiref: str,
        target_uideps: Uideps,
        uiref_to_newconf_map: UirefToConfMap,
        forced: bool = False) -> list[IDgConfBuilder]:

    if not target_uideps.test_contains(target_uiref, {uitype.UI_TYPE_MEDIASTORE}):
        return []

    try:
        uimod_ms = target_uideps.find_any({uitype.UI_TYPE_MEDIASTORE})
    except NotFoundError as x:
        eventlog.log_w(f'fail: link absent ({x})')
        return []

    activate = forced

    if not activate:
        return []

    return [_MswCreator(uimod_ms)]


class _MswCreator(IDgConfBuilder):
    def __init__(self, ms: IUiConfProvider) -> None:
        self._dgmod_bsw: Final[dg.DgMod] = dg.DgMod(dg.EDGE_TYPE_BLOBSTOREW, ms.id)

        self._uimod_ms: Final[IUiConfProvider] = ms

        self._ss = None

    @property
    def dgmod(self) -> dg.DgMod:
        return self._dgmod_bsw

    def setup(self, uiref_to_conf_map: UirefToConfMap) -> None:
        pass

    def build(self) -> None:
        edge_lbs = self._dgmod_bsw
        edge_lbs.install()
        edge_lbs.save_setupstore_conf(active=True)

    def notify(self) -> None:
        self.dgmod.sigkill()


class _MswDeleter(IDgConfDeleter):
    def __init__(self, uimod_ms: IUiConfProvider) -> None:
        self._dgmod_bsw: Final[dg.DgMod] = dg.DgMod(dg.EDGE_TYPE_BLOBSTOREW, uimod_ms.id)

    def run(self) -> None:
        self._dgmod_bsw.uninstall()

def msw_create_dgconf_deleter(target_uimod: IUiConfProvider) -> List[IDgConfDeleter]:
    if target_uimod.type == uitype.UI_TYPE_MEDIASTORE:
        return [_MswDeleter(target_uimod)]
    return []
